@@ -1,3 +1,6 @@
+;; Copyright (c) Cognitect, Inc.
+;; All rights reserved.
+
 (ns ^:skip-wiki cognitect.aws.http.cognitect
   (:require [cognitect.http-client :as impl]
             [cognitect.aws.http :as aws]))
@@ -6,7 +9,7 @@
 
 (defn create
   []
-  (let [c (impl/create {})]
+  (let [c (impl/create {:follow-redirects false})]
     (reify aws/HttpClient
       (-submit [_ request channel]
         (impl/submit c request channel))
