@@ -3,10 +3,10 @@
 
 (ns ^:skip-wiki cognitect.aws.protocols.ec2
   "Impl, don't call directly."
-  (:require [cognitect.aws.util :as util]
-            [cognitect.aws.client :as client]
+  (:require [cognitect.aws.protocols :as aws.protocols]
+            [cognitect.aws.protocols.query :as query]
             [cognitect.aws.shape :as shape]
-            [cognitect.aws.protocols.query :as query]))
+            [cognitect.aws.util :as util]))
 
 (set! *warn-on-reflection* true)
 
@@ -18,36 +18,36 @@
       default))
 
 (defmulti serialize
-  (fn [shape args serialized prefix] (:type shape)))
+  (fn [_shapes shape _args _serialized _prefix] (:type shape)))
 
 (defmethod serialize :default
-  [shape args serialized prefix]
-  (query/serialize shape args serialized prefix))
+  [shapes shape args serialized prefix]
+  (query/serialize shapes shape args serialized prefix))
 
 (defmethod serialize "structure"
-  [shape args serialized prefix]
+  [shapes shape args serialized prefix]
   (let [args (util/with-defaults shape args)]
     (reduce (fn [serialized k]
-              (let [member-shape (shape/member-shape shape k)
+              (let [member-shape (shape/resolve shapes (shape/structure-member-shape-ref shape k))
                     member-name  (serialized-name member-shape (name k))]
                 (if (contains? args k)
-                  (serialize member-shape (k args) serialized (conj prefix member-name))
+                  (serialize shapes member-shape (k args) serialized (conj prefix member-name))
                   serialized)))
             serialized
             (keys (:members shape)))))
 
 (defmethod serialize "list"
-  [shape args serialized prefix]
-  (let [member-shape (shape/list-member-shape shape)]
+  [shapes shape args serialized prefix]
+  (let [member-shape (shape/resolve shapes (shape/list-member-shape-ref shape))]
     (reduce (fn [serialized [i member]]
-              (serialize member-shape member serialized (conj prefix (str i))))
+              (serialize shapes member-shape member serialized (conj prefix (str i))))
             serialized
             (map-indexed (fn [i member] [(inc i) member]) args))))
 
-(defmethod client/build-http-request "ec2"
+(defmethod aws.protocols/build-http-request "ec2"
   [service op-map]
-  (query/build-query-http-request serialize service op-map))
+  (query/build-query-http-request op-map service serialize))
 
-(defmethod client/parse-http-response "ec2"
-  [service {:keys [op] :as op-map} {:keys [status body] :as http-response}]
+(defmethod aws.protocols/parse-http-response "ec2"
+  [service op-map http-response]
   (query/build-query-http-response service op-map http-response))
