@@ -5,7 +5,6 @@
   "Impl, don't call directly."
   (:require [clojure.string :as str]
             [cognitect.aws.protocols :as aws.protocols]
-            [cognitect.aws.service :as service]
             [cognitect.aws.shape :as shape]
             [cognitect.aws.util :as util]))
 
@@ -20,73 +19,75 @@
   (or (:locationName shape)
       default))
 
-(defmulti serialize
-  (fn [shape _args _serialized _prefix] (:type shape)))
-
 (defn prefix-assoc
   [serialized prefix val]
   (assoc serialized (str/join "." prefix) val))
 
+(defmulti serialize
+  (fn [_shapes shape _args _serialized _prefix] (:type shape)))
+
 (defmethod serialize :default
-  [_shape args serialized prefix]
+  [_shapes _shape args serialized prefix]
   (prefix-assoc serialized prefix (str args)))
 
 (defmethod serialize "structure"
-  [shape args serialized prefix]
+  [shapes shape args serialized prefix]
   (let [args (util/with-defaults shape args)]
     (reduce (fn [serialized k]
-              (let [member-shape (shape/member-shape shape k)
+              (let [member-shape (shape/resolve shapes (shape/structure-member-shape-ref shape k))
                     member-name  (serialized-name member-shape (name k))]
                 (if (contains? args k)
-                  (serialize member-shape (k args) serialized (conj prefix member-name))
+                  (serialize shapes member-shape (k args) serialized (conj prefix member-name))
                   serialized)))
             serialized
             (keys (:members shape)))))
 
 (defmethod serialize "list"
-  [shape args serialized prefix]
+  [shapes shape args serialized prefix]
   (if (empty? args)
     (prefix-assoc serialized prefix "")
-    (let [member-shape (shape/list-member-shape shape)
+    (let [member-shape (shape/resolve shapes (shape/list-member-shape-ref shape))
           list-prefix (if (:flattened shape)
                         (conj (vec (butlast prefix)) (serialized-name member-shape (last prefix)))
                         (conj prefix (serialized-name member-shape "member")))]
       (reduce (fn [serialized [i member]]
-                (serialize member-shape member serialized (conj list-prefix (str i))))
+                (serialize shapes member-shape member serialized (conj list-prefix (str i))))
               serialized
               (map-indexed (fn [i member] [(inc i) member]) args)))))
 
 (defmethod serialize "map"
-  [shape args serialized prefix]
+  [shapes shape args serialized prefix]
   (let [map-prefix (if (:flattened shape) prefix (conj prefix "entry"))
-        key-shape (shape/key-shape shape)
+        key-shape (shape/resolve shapes (shape/map-key-shape-ref shape))
         key-suffix (serialized-name key-shape "key")
-        value-shape (shape/value-shape shape)
+        value-shape (shape/resolve shapes (shape/map-value-shape-ref shape))
         value-suffix (serialized-name value-shape "value")]
     (reduce (fn [serialized [i k v]]
               (as-> serialized $
-                (serialize key-shape (name k) $ (conj map-prefix (str i) key-suffix))
-                (serialize value-shape v $ (conj map-prefix (str i) value-suffix))))
+                (serialize shapes key-shape (name k) $ (conj map-prefix (str i) key-suffix))
+                (serialize shapes value-shape v $ (conj map-prefix (str i) value-suffix))))
             serialized
             (map-indexed (fn [i [k v]] [(inc i) k v]) args))))
 
 (defmethod serialize "blob"
-  [_shape args serialized prefix]
+  [_shapes _shape args serialized prefix]
   (prefix-assoc serialized prefix (util/base64-encode args)))
 
-(defmethod serialize "timestamp" [shape args serialized prefix]
+(defmethod serialize "timestamp"
+  [_shapes shape args serialized prefix]
   (prefix-assoc serialized prefix (shape/format-date shape
                                                      args
                                                      (partial util/format-date util/iso8601-date-format))))
 
 (defmethod serialize "boolean"
-  [_shape args serialized prefix]
+  [_shapes _shape args serialized prefix]
   (prefix-assoc serialized prefix (if args "true" "false")))
 
 (defn build-query-http-request
-  [serialize service {:keys [op request]}]
+  [{:keys [op request]} service serialize]
   (let [operation   (get-in service [:operations op])
-        input-shape (service/shape service (:input operation))
+        shapes      (:shapes service)
+        input-shape (shape/resolve shapes (:input operation))
         params      {"Action"  (name op)
                      "Version" (get-in service [:metadata :apiVersion])}]
     {:request-method :post
@@ -95,17 +96,18 @@
      :uri            "/"
      :headers        (aws.protocols/headers service operation)
      :body           (util/query-string
-                      (serialize input-shape request params []))}))
+                      (serialize shapes input-shape request params []))}))
 
 (defmethod aws.protocols/build-http-request "query"
-  [service req-map]
-  (build-query-http-request serialize service req-map))
+  [service op-map]
+  (build-query-http-request op-map service serialize))
 
 (defn build-query-http-response
   [service {:keys [op]} {:keys [body]}]
-  (let [operation (get-in service [:operations op])]
-    (if-let [output-shape (service/shape service (:output operation))]
-      (shape/xml-parse output-shape (util/bbuf->str body))
+  (let [operation (get-in service [:operations op])
+        shapes    (:shapes service)]
+    (if-let [output-shape (shape/resolve shapes (:output operation))]
+      (shape/xml-parse shapes output-shape (util/bbuf->str body))
       (util/xml->map (util/xml-read (util/bbuf->str body))))))
 
 (defmethod aws.protocols/parse-http-response "query"
