@@ -16,7 +16,7 @@
 ;(defn method-sig [^java.lang.reflect.Method meth]
 ;  [(. meth (getName)) (seq (. meth (getParameterTypes)))])
 
-(defn- non-private-methods [^Class c]
+(defn- filter-methods [^Class c invalid-method?]
   (loop [mm {}
          considered #{}
          c c]
@@ -32,17 +32,30 @@
                       mods (. meth (getModifiers))
                       mk (method-sig meth)]
                   (if (or (considered mk)
-                          (not (or (Modifier/isPublic mods) (Modifier/isProtected mods)))
-                          ;(. Modifier (isPrivate mods))
-                          (. Modifier (isStatic mods))
-                          (. Modifier (isFinal mods))
-                          (= "finalize" (.getName meth)))
+                          (invalid-method? meth))
                     (recur mm (conj considered mk) (next meths))
                     (recur (assoc mm mk meth) (conj considered mk) (next meths))))
                 [mm considered]))]
         (recur mm considered (. c (getSuperclass))))
       mm)))
 
+(defn- non-private-methods [^Class c]
+  (let [not-overridable? (fn [^java.lang.reflect.Method meth]
+                           (let [mods (. meth (getModifiers))]
+                             (or (not (or (Modifier/isPublic mods) (Modifier/isProtected mods)))
+                                 (. Modifier (isStatic mods))
+                                 (. Modifier (isFinal mods))
+                                 (= "finalize" (.getName meth)))))]
+    (filter-methods c not-overridable?)))
+
+(defn- protected-final-methods [^Class c]
+  (let [not-exposable? (fn [^java.lang.reflect.Method meth]
+                         (let [mods (. meth (getModifiers))]
+                           (not (and (Modifier/isProtected mods)
+                                     (Modifier/isFinal mods)
+                                     (not (Modifier/isStatic mods))))))]
+    (filter-methods c not-exposable?)))
+
 (defn- ctor-sigs [^Class super]
   (for [^Constructor ctor (. super (getDeclaredConstructors))
         :when (not (. Modifier (isPrivate (. ctor (getModifiers)))))]
@@ -71,24 +84,43 @@
 
 (def ^{:private true} prim->class
      {'int Integer/TYPE
+      'ints (Class/forName "[I")
       'long Long/TYPE
+      'longs (Class/forName "[J")
       'float Float/TYPE
+      'floats (Class/forName "[F")
       'double Double/TYPE
+      'doubles (Class/forName "[D")
       'void Void/TYPE
       'short Short/TYPE
+      'shorts (Class/forName "[S")
       'boolean Boolean/TYPE
+      'booleans (Class/forName "[Z")
       'byte Byte/TYPE
-      'char Character/TYPE})
+      'bytes (Class/forName "[B")
+      'char Character/TYPE
+      'chars (Class/forName "[C")})
 
-(defn- ^Class the-class [x] 
-  (cond 
-   (class? x) x
-   (contains? prim->class x) (prim->class x)
-   :else (let [strx (str x)]
-           (clojure.lang.RT/classForName 
-            (if (some #{\. \[} strx)
-              strx
-              (str "java.lang." strx))))))
+(defn- the-array-class [sym]
+  (clojure.lang.RT/classForName
+   (let [cn (namespace sym)]
+     (clojure.lang.Compiler$HostExpr/buildArrayClassDescriptor
+      (if (or (clojure.lang.Compiler/primClass (symbol cn)) (some #{\.} cn))
+        sym
+        (symbol (str "java.lang." cn) (name sym)))))))
+
+(defn- ^Class the-class [x]
+  (cond
+    (class? x) x
+    (symbol? x) (cond (contains? prim->class x) (prim->class x)
+                      (clojure.lang.Compiler$HostExpr/looksLikeArrayClass x)
+                        (the-array-class x)
+                      :else (let [strx (str x)]
+                              (clojure.lang.RT/classForName
+                               (if (some #{\. \[} strx)
+                                 strx
+                                 (str "java.lang." strx)))))
+    :else (clojure.lang.RT/classForName x)))
 
 ;; someday this can be made codepoint aware
 (defn- valid-java-method-name
@@ -112,7 +144,7 @@
         interfaces (map the-class implements)
         supers (cons super interfaces)
         ctor-sig-map (or constructors (zipmap (ctor-sigs super) (ctor-sigs super)))
-        cv (new ClassWriter (. ClassWriter COMPUTE_MAXS))
+        cv (clojure.lang.Compiler/classWriter)
         cname (. name (replace "." "/"))
         pkg-name name
         impl-pkg-name (str impl-ns)
@@ -140,10 +172,11 @@
         ifn-type (totype clojure.lang.IFn)
         iseq-type (totype clojure.lang.ISeq)
         ex-type  (totype java.lang.UnsupportedOperationException)
+        util-type (totype clojure.lang.Util)
         all-sigs (distinct (concat (map #(let[[m p] (key %)] {m [p]}) (mapcat non-private-methods supers))
                                    (map (fn [[m p]] {(str m) [p]}) methods)))
         sigs-by-name (apply merge-with concat {} all-sigs)
-        overloads (into {} (filter (fn [[m s]] (next s)) sigs-by-name))
+        overloads (into1 {} (filter (fn [[m s]] (next s)) sigs-by-name))
         var-fields (concat (when init [init-name]) 
                            (when post-init [post-init-name])
                            (when main [main-name])
@@ -232,7 +265,7 @@
             (. gen (endMethod))))
         ]
                                         ;start class definition
-    (. cv (visit (. Opcodes V1_5) (+ (. Opcodes ACC_PUBLIC) (. Opcodes ACC_SUPER))
+    (. cv (visit (. Opcodes V1_8) (+ (. Opcodes ACC_PUBLIC) (. Opcodes ACC_SUPER))
                  cname nil (iname super)
                  (when-let [ifc (seq interfaces)]
                    (into-array (map iname ifc)))))
@@ -266,11 +299,9 @@
         (. gen putStatic ctype (var-name v) var-type))
       
       (when load-impl-ns
-        (. gen push "clojure.core")
-        (. gen push "load")
-        (. gen (invokeStatic rt-type (. Method (getMethod "clojure.lang.Var var(String,String)"))))
         (. gen push (str "/" impl-cname))
-        (. gen (invokeInterface ifn-type (new Method "invoke" obj-type (to-types [Object]))))
+        (. gen push ctype)
+        (. gen (invokeStatic util-type (. Method (getMethod "Object loadWithClass(String,Class)"))))
 ;        (. gen push (str (.replace impl-pkg-name \- \_) "__init"))
 ;        (. gen (invokeStatic class-type (. Method (getMethod "Class forName(String)"))))
         (. gen pop))
@@ -280,13 +311,15 @@
     
                                         ;ctors
     (doseq [[pclasses super-pclasses] ctor-sig-map]
-      (let [pclasses (map the-class pclasses)
+      (let [constructor-annotations (meta pclasses)
+            pclasses (map the-class pclasses)
             super-pclasses (map the-class super-pclasses)
             ptypes (to-types pclasses)
             super-ptypes (to-types super-pclasses)
             m (new Method "<init>" (. Type VOID_TYPE) ptypes)
             super-m (new Method "<init>" (. Type VOID_TYPE) super-ptypes)
             gen (new GeneratorAdapter (. Opcodes ACC_PUBLIC) m nil nil cv)
+            _ (add-annotations gen constructor-annotations)
             no-init-label (. gen newLabel)
             end-label (. gen newLabel)
             no-post-init-label (. gen newLabel)
@@ -310,7 +343,7 @@
                                                   (arg-types (count ptypes)))))
                                         ;expecting [[super-ctor-args] state] returned
             (. gen dup)
-            (. gen push 0)
+            (. gen push (int 0))
             (. gen (invokeStatic rt-type nth-method))
             (. gen storeLocal local)
             
@@ -318,14 +351,14 @@
             (. gen dupX1)
             (dotimes [i (count super-pclasses)]
               (. gen loadLocal local)
-              (. gen push i)
+              (. gen push (int i))
               (. gen (invokeStatic rt-type nth-method))
               (. clojure.lang.Compiler$HostExpr (emitUnboxArg nil gen (nth super-pclasses i))))
             (. gen (invokeConstructor super-type super-m))
             
             (if state
               (do
-                (. gen push 1)
+                (. gen push (int 1))
                 (. gen (invokeStatic rt-type nth-method))
                 (. gen (putField ctype state-name obj-type)))
               (. gen pop))
@@ -391,7 +424,7 @@
                                                                (. m (getName))
                                                                (. m (getDescriptor)))))))
                                         ;add methods matching interfaces', if no fn -> throw
-      (reduce (fn [mm ^java.lang.reflect.Method meth]
+      (reduce1 (fn [mm ^java.lang.reflect.Method meth]
                 (if (contains? mm (method-sig meth))
                   mm
                   (do
@@ -404,10 +437,11 @@
          (emit-forwarding-method mname pclasses rclass (:static (meta msig))
                                  emit-unsupported))
                                         ;expose specified overridden superclass methods
-       (doseq [[local-mname ^java.lang.reflect.Method m] (reduce (fn [ms [[name _ _] m]]
+       (doseq [[local-mname ^java.lang.reflect.Method m] (reduce1 (fn [ms [[name _ _] m]]
                               (if (contains? exposes-methods (symbol name))
                                 (conj ms [((symbol name) exposes-methods) m])
-                                ms)) [] (seq mm))]
+                                ms)) [] (concat (seq mm)
+                                                (seq (protected-final-methods super))))]
          (let [ptypes (to-types (.getParameterTypes m))
                rtype (totype (.getReturnType m))
                exposer-m (new Method (str local-mname) rtype ptypes)
@@ -545,7 +579,7 @@
   the first argument, followed by the arguments to the constructor.
   It will be called every time an object of this class is created,
   immediately after all the inherited constructors have completed.
-  It's return value is ignored.
+  Its return value is ignored.
 
   :methods [ [name [param-types] return-type], ...]
 
@@ -610,7 +644,7 @@
   
   [& options]
     (when *compile-files*
-      (let [options-map (into {} (map vec (partition 2 options)))
+      (let [options-map (into1 {} (map vec (partition 2 options)))
             [cname bytecode] (generate-class options-map)]
         (clojure.lang.Compiler/writeClassFile cname bytecode))))
 
@@ -623,25 +657,30 @@
   fully-qualified class name given as a string or symbol
   (such as 'java.lang.String)"
   [c]
-  (if (or (instance? Class c) (prim->class c))
-    (Type/getType (the-class c))
-    (let [strx (str c)]
-      (Type/getObjectType 
-       (.replace (if (some #{\.} strx)
-                   strx
-                   (str "java.lang." strx)) 
-                 "." "/")))))
+  (let [c (or (and (symbol? c) (clojure.lang.Compiler$HostExpr/maybeArrayClass c)) c)]
+    (if (or (instance? Class c) (prim->class c))
+      (Type/getType (the-class c))
+      (let [strx (str c)]
+        (Type/getObjectType
+         (.replace (if (some #{\. \[} strx)
+                     strx
+                     (str "java.lang." strx))
+                   "." "/"))))))
 
 (defn- generate-interface
   [{:keys [name extends methods]}]
+  (when (some #(-> % first clojure.core/name (.contains "-")) methods)
+    (throw
+      (IllegalArgumentException. "Interface methods must not contain '-'")))
   (let [iname (.replace (str name) "." "/")
-        cv (ClassWriter. ClassWriter/COMPUTE_MAXS)]
-    (. cv visit Opcodes/V1_5 (+ Opcodes/ACC_PUBLIC 
+        cv (clojure.lang.Compiler/classWriter)]
+    (. cv visit Opcodes/V1_8 (+ Opcodes/ACC_PUBLIC 
                                 Opcodes/ACC_ABSTRACT
                                 Opcodes/ACC_INTERFACE)
        iname nil "java/lang/Object"
        (when (seq extends)
          (into-array (map #(.getInternalName (asm-type %)) extends))))
+    (when (not= "NO_SOURCE_FILE" *source-path*) (. cv visitSource *source-path* nil))
     (add-annotations cv (meta name))
     (doseq [[mname pclasses rclass pmetas] methods]
       (let [mv (. cv visitMethod (+ Opcodes/ACC_PUBLIC Opcodes/ACC_ABSTRACT)
@@ -690,10 +729,10 @@
   [& options]
     (let [options-map (apply hash-map options)
           [cname bytecode] (generate-interface options-map)]
-      (if *compile-files*
-        (clojure.lang.Compiler/writeClassFile cname bytecode)
-        (.defineClass ^DynamicClassLoader (deref clojure.lang.Compiler/LOADER) 
-                      (str (:name options-map)) bytecode options)))) 
+      (when *compile-files*
+        (clojure.lang.Compiler/writeClassFile cname bytecode))
+      (.defineClass ^DynamicClassLoader (deref clojure.lang.Compiler/LOADER)
+                    (str (:name options-map)) bytecode options)))
 
 (comment
 
