@@ -24,7 +24,7 @@
     [java.io File InputStreamReader BufferedReader]
     [java.lang ProcessBuilder ProcessBuilder$Redirect]
     [java.util List]
-    [java.util.concurrent ExecutorService]))
+    [java.util.concurrent ConcurrentHashMap ExecutorService]))
 
 ;(set! *warn-on-reflection* true)
 
@@ -227,9 +227,8 @@
          :cut' (assoc cut [lib coord-id] coord-excl)
          :child-pred (fn [lib] (not (contains? coord-excl lib)))})
 
-      ;; if seeing same lib/ver again, narrow exclusions to intersection of prior and new.
-      ;; only include new unexcluded children (old excl set minus new excl set)
-      ;; as others were already enqueued when first added
+      ;; if seeing same lib/ver again, narrow exclusions to intersection of prior and new,
+      ;; must reconsider previously included children as prev parent may get omitted
       (= reason :same-version)
       (let [exclusions' (if (seq coord-excl) (assoc exclusions use-path coord-excl) exclusions)
             cut-coord (get cut [lib coord-id]) ;; previously cut from this lib, so were not enqueued
@@ -238,6 +237,7 @@
         {:exclusions' exclusions'
          :cut' (assoc cut [lib coord-id] new-cut)
          :child-pred (set enq-only)})
+         ;:child-pred (fn [lib] (not (contains? new-cut lib)))})
 
       :else ;; otherwise, no change
       {:exclusions' exclusions, :cut' cut})))
@@ -283,16 +283,21 @@
   "Is any part of the parent path missing from the selected lib/versions?
   This can happen if a newer version was found, orphaning previously selected children."
   [vmap parent-path]
-  (when (seq parent-path)
-    (loop [path parent-path]
-      (if (seq path)
-        (let [lib (last path)
-              check-path (vec (butlast path))
-              {:keys [paths select]} (get vmap lib)]
-          (if (contains? (get paths select) check-path)
-            (recur check-path)
-            true))
-        false))))
+  (loop [path parent-path
+         more-paths nil]
+    (if (seq path)
+      (let [lib (last path)
+            check-path (vec (butlast path))
+            {:keys [paths select]} (get vmap lib)]
+        (let [paths-to-selected (get paths select)]
+          (if (contains? paths-to-selected check-path)
+            ;; add alternative paths to root that include the selected lib
+            (recur check-path (concat more-paths (remove #(= % check-path) paths-to-selected)))
+            (if (seq more-paths)
+              ;; consider alternative paths before considering lib to be omitted
+              (recur (first more-paths) (rest more-paths))
+              true))))
+      false)))
 
 (defn- deselect-orphans
   "For the given paths, deselect any libs whose only selected version paths are in omitted-paths"
@@ -394,44 +399,50 @@
 (defn- expand-deps
   "Dep tree expansion, returns version map"
   [deps default-deps override-deps config executor trace?]
-  (letfn [(err-handler [throwable]
-            (do
-              (concurrent/shutdown-on-error executor)
-              (throw ^Throwable throwable)))
-          (children-task [lib use-coord use-path child-pred]
-            {:pend-children
-             (let [{:deps/keys [manifest root]} use-coord]
-               (dir/with-dir (if root (jio/file root) dir/*the-dir*)
-                 (concurrent/submit-task executor
-                   #(try
-                      (canonicalize-deps (ext/coord-deps lib use-coord manifest config) config)
-                      (catch Throwable t t)))))
-             :ppath use-path
-             :child-pred child-pred})]
-    (loop [pendq nil ;; a resolved child-lookup thunk to look at first
-           q (into PersistentQueue/EMPTY (map vector deps)) ;; queue of nodes or child-lookups
-           version-map nil ;; track all seen versions of libs and which version is selected
-           exclusions nil ;; tracks exclusions marked in the tree
-           cut nil ;; tracks cuts made of child nodes based on exclusions
-           trace []] ;; trace expansion
-      (let [{:keys [path pendq q']} (next-path pendq q err-handler)]
-        (if path
-          (let [[lib coord] (peek path)
-                parents (pop path)
-                use-path (conj parents lib)
-                override-coord (get override-deps lib)
-                choose-coord (cond override-coord override-coord
-                                   coord coord
-                                   :else (get default-deps lib))
-                use-coord (merge choose-coord (ext/manifest-type lib choose-coord config))
-                coord-id (ext/dep-id lib use-coord config)
-                {:keys [include reason vmap]} (include-coord? version-map lib use-coord coord-id parents exclusions config)
-                ;_ (println "loop" lib coord-id "include=" include "reason=" reason)
-                {:keys [exclusions' cut' child-pred]} (update-excl lib use-coord coord-id use-path include reason exclusions cut)
-                new-q (if child-pred (conj q' (children-task lib use-coord use-path child-pred)) q')]
-            (recur pendq new-q vmap exclusions' cut'
-              (trace+ trace? trace parents lib coord use-coord coord-id override-coord include reason)))
-          (cond-> version-map trace? (with-meta {:trace {:log trace, :vmap version-map, :exclusions exclusions}})))))))
+  (let [memoized-deps (ConcurrentHashMap. 100)]
+    (letfn [(err-handler [throwable]
+              (do
+                (concurrent/shutdown-on-error executor)
+                (throw ^Throwable throwable)))
+            (children-task [lib use-coord use-path child-pred]
+              {:pend-children
+               (let [{:deps/keys [manifest root]} use-coord]
+                 (dir/with-dir (if root (jio/file root) dir/*the-dir*)
+                   (concurrent/submit-task executor
+                     #(try
+                        (let [k [lib use-coord]]
+                          (or
+                            (.get memoized-deps k)
+                            (let [child-deps (canonicalize-deps (ext/coord-deps lib use-coord manifest config) config)]
+                              (.putIfAbsent memoized-deps k child-deps)
+                              child-deps)))
+                        (catch Throwable t t)))))
+               :ppath use-path
+               :child-pred child-pred})]
+      (loop [pendq nil ;; a resolved child-lookup thunk to look at first
+             q (into PersistentQueue/EMPTY (map vector deps)) ;; queue of nodes or child-lookups
+             version-map nil ;; track all seen versions of libs and which version is selected
+             exclusions nil ;; tracks exclusions marked in the tree
+             cut nil ;; tracks cuts made of child nodes based on exclusions
+             trace []] ;; trace expansion
+        (let [{:keys [path pendq q']} (next-path pendq q err-handler)]
+          (if path
+            (let [[lib coord] (peek path)
+                  parents (pop path)
+                  use-path (conj parents lib)
+                  override-coord (get override-deps lib)
+                  choose-coord (cond override-coord override-coord
+                                     coord coord
+                                     :else (get default-deps lib))
+                  use-coord (merge choose-coord (ext/manifest-type lib choose-coord config))
+                  coord-id (ext/dep-id lib use-coord config)
+                  {:keys [include reason vmap]} (include-coord? version-map lib use-coord coord-id parents exclusions config)
+                  ; _ (println "loop" lib coord-id "include=" include "reason=" reason)
+                  {:keys [exclusions' cut' child-pred]} (update-excl lib use-coord coord-id use-path include reason exclusions cut)
+                  new-q (if child-pred (conj q' (children-task lib use-coord use-path child-pred)) q')]
+              (recur pendq new-q vmap exclusions' cut'
+                (trace+ trace? trace parents lib coord use-coord coord-id override-coord include reason)))
+            (cond-> version-map trace? (with-meta {:trace {:log trace, :vmap version-map, :exclusions exclusions}}))))))))
 
 (defn- cut-orphans
   "Remove any selected lib that does not have a selected parent path"
@@ -630,8 +641,9 @@
   (-> (make-classpath-map {:paths paths} lib-map classpath-args) :classpath-roots join-classpath))
 
 (defn tool
-  "Transform project edn for tool by applying tool args (keys = :paths, :deps) and
-  returning an updated project edn."
+  "Replaces project-edn :deps and :paths with tool :deps and :paths
+  and returns a new project-edn map.
+  (Also supports :replace-deps and :replace-paths as aliases.)"
   [project-edn tool-args]
   (let [{:keys [replace-deps replace-paths deps paths]} tool-args]
     (cond-> project-edn
@@ -704,8 +716,8 @@
                      (dir/with-dir root-dir
                        (let [basis (create-basis
                                     {:project :standard ;; deps.edn at root
-                                     :extra {:aliases {:deps/TOOL {:replace-deps {} :replace-paths ["."]}}}
-                                     :aliases [:deps/TOOL alias]})
+                                     :args {:replace-deps {} :replace-paths ["."]}
+                                     :aliases [alias]})
                              cp (join-classpath (:classpath-roots basis))
                              qual-f (qualify-fn f (get-in basis [:aliases alias]))
                              exit (exec-prep! root-dir cp qual-f exec-args)]
@@ -795,8 +807,9 @@
    Each dep source value can be :standard, a string path, a deps edn map, or nil.
    Sources are merged in the order - :root, :user, :project, :extra.
 
-   Aliases refer to argmaps in the merged deps that will be supplied to the basis
-   subprocesses (tool, resolve-deps, make-classpath-map).
+   When resolving deps and forming the classpath, different steps need various
+   args. These are pulled from the 'argmap' which is a merged set of args
+   selected by the aliases and optionally a map of args.
 
    Options:
      :dir     - directory root path, defaults to current directory
@@ -805,6 +818,7 @@
      :project - dep source, default = :standard (\"deps.edn\")
      :extra   - dep source, default = nil
      :aliases - coll of aliases, default = nil
+     :args    - extra map of argmap args, merged after aliases
 
    The following subprocess argmap args can be provided:
      Key                  Subproc             Description
@@ -822,13 +836,9 @@
     :libs - lib map, per resolve-deps
     :classpath - classpath map per make-classpath-map
     :classpath-roots - vector of paths in classpath order"
-  [{:keys [dir root user project extra aliases] :as params}]
+  [{:keys [dir root user project extra aliases args] :as params}]
   (dir/with-dir (jio/file (or dir "."))
-    (let [basis-config (cond-> nil
-                         (contains? params :root) (assoc :root root)
-                         (contains? params :project) (assoc :project project)
-                         (contains? params :user) (assoc :user user)
-                         (contains? params :extra) (assoc :extra extra)
+    (let [basis-config (cond-> (select-keys params [:root :project :user :extra :args])
                          (seq aliases) (assoc :aliases (vec aliases)))
 
           {root-edn :root user-edn :user project-edn :project extra-edn :extra} (create-edn-maps params)
@@ -841,11 +851,13 @@
           argmap-data (->> aliases
                         (remove nil?)
                         (map #(get alias-data %)))
-          argmap (apply merge-alias-maps argmap-data)
+          argmap (apply merge-alias-maps (concat argmap-data [args]))
 
           project-tooled-edn (tool project-edn argmap)
           merged-edn (merge-edns [root-edn user-edn project-tooled-edn extra-edn])
-          basis (calc-basis merged-edn {:resolve-args argmap, :classpath-args argmap})]
+          basis (if (:skip-cp argmap) ;; UNSUPPORTED, USE AT YOUR OWN RISK
+                  (assoc merged-edn :argmap argmap)
+                  (calc-basis merged-edn {:resolve-args argmap, :classpath-args argmap}))]
       (cond-> (assoc basis :basis-config basis-config)
         (pos? (count argmap)) (assoc :argmap argmap)))))
 
@@ -941,6 +953,9 @@
                :mvn/repos (merge mvn/standard-repos {"datomic-cloud" {:url "s3://datomic-releases-1fc2183a/maven/releases"}})}
     nil)
 
+  (create-basis
+    {:user {} :project {} :aliases [:deps] :args {:ns-default 'foo}})
+
   (print-tree
     (resolve-deps {:deps {'org.clojure/clojure {:mvn/version "1.8.0"}
                           'org.clojure/core.memoize {:mvn/version "0.5.8"}}
